/*  Denys VK3ZYZ (with help from Google and others)
  CTCSS generator
*/

#define Version "CTCSS_20210207"

int     timer       =  100;       // sub tone setting. The higher the number, the slower the timing.
int                     i ;
int     SineDivider = 4065;       // default sine wave divider
int     CTCSS       =    0;
int     TRXmode     =    0;

#define On  1
#define Off 0
#define RefFrequ      500000        // 16Mhz / 32 steps for CTCSS sine wave generator

#define TRX           2           // input HIGH = TX, LOW = RX.
#define Input1        3           // CTCSS_select1
#define Input2        4           // CTCSS_select2
#define Input3        5           // CTCSS_select3
#define Input4        6           // CTCSS_select4
#define Input5        7           // CTCSS_select5

/* ********************* PortB0-PB4 used to generate CTCSS tones via R2R nerwork.
#define spare1         8           // Sub Tone R2R b0 PB0
#define spare2         9           // Sub Tone R2R b1 PB1
#define spare3        10           // Sub Tone R2R b2 PB2
#define spare4        11           // Sub Tone R2R b3 PB3
#define spare5        12           // Sub Tone R2R b4 PB4
*/

float SubTone [] // Edit this for CTCSS frequencies wanted. Selected by Input1-Input5 settings.
{   0.0,
   67.0,   
   69.3,
   71.9, 
   74.4,
   77.0,  
   79.7,
   82.5,   
   85.4,
   88.5,
   91.5,
   94.8,
   97.4,
  100.0,
  103.5,
  107.2,   
  110.9,
  114.8,
  118.1,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,  
};

int sinetable[] // one cycle sine wave via R2R network.
{ 0,1,2,3,5,7,10,13,16,19,22,25,27,28,29,30,31,30,29,28,27,25,22,19,16,13,10,7,5,3,2,1, };


void setup() {

  pinMode(TRX,    INPUT_PULLUP);     // Hi = TX, LOW =  RX
  pinMode(Input1, INPUT_PULLUP);     
  pinMode(Input2, INPUT_PULLUP);
  pinMode(Input3, INPUT_PULLUP);   
  pinMode(Input4, INPUT_PULLUP);
  pinMode(Input5, INPUT_PULLUP);

  DDRB  = B011111;                // sets Arduino pins 8 to 12 as outputs.
  PORTB = B010000;                // R2R out at rest to half level.

}

/******************************
  main loop
*******************************/
void loop() {
  int freq = 0;              // What CTCSS frequencty is selected?
  if (digitalRead(Input1==HIGH))
    { freq = freq +1;
    }
  if (digitalRead(Input2==HIGH))
    { freq = freq +2;
    }   
  if (digitalRead(Input3==HIGH))
    { freq = freq +4;
    }
  if (digitalRead(Input4==HIGH))
    { freq = freq +8;
    }
  if (digitalRead(Input5==HIGH))
    { freq = freq +16;
    }
  
  if (digitalRead(TRX) !=TRXmode) 
  {                     // Transmitting......
    TRXmode=(digitalRead(TRX));   
      if ((SubTone[freq]) != 0) // if the CTCSS frequ in not zero..
      { SineDivider = (RefFrequ/(SubTone[freq])); // get the CTCSS frequ needed.
      CTCSS_Tone();     // Turn CTCSS on for this channel.
      }
      else 
      {
      TIMSK1 = 0;       // Turn CTCSS off for this channel.
      } 
   }
   else
   { TIMSK1 = 0;        // Turn CTCSS off for receive.
   } 
}

void CTCSS_Tone() // This is the Timer 1 interupt routine to generate the tone.
{
  cli();                    // interupts off for setup
  TCCR1A = 0;               // 
  TCCR1B = 0;               //  no prescaler
                            // set compare match register for 1hz increments
  OCR1A = SineDivider ;     // = (16*10^6) / (1*1024) - 1 (must be <65536)                       
  TCCR1B |= (1 << WGM12);   // turn on CTC mode                         
  TCCR1B |= (0 << CS12) | (0 << CS11) | (1 << CS10); // Set CS10 no prescaler                         
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
  sei();                    // Interupts on.       
}

ISR(TIMER1_COMPA_vect)      // Each time the Timer 1 overflows,
{   PORTB=(sinetable[i++]); // write the next value from table to PORTB
    if(i>=32)               // Has the end of the sinewave table been reached?
    { i=0;                  // Yes, loop back to the start of the table.
    }
}


